/*HEADER*******************************************************************
***************************************************************************
***
*** Copyright (c) 2008 Vision Components
***
*** All rights reserved
***
*** This software embodies materials and concepts which are confidential
*** to Vision Components
***
*** File: vccolor.c
***                                                            
*** Comments: 
*** 
***	   Test of OpenCV color processing
***
*** Revision History:
***    Date          Version    Author  Changes
***    Oct 01, 2008  3.03       ME      Initial Version
***
***************************************************************************
*END***********************************************************************/

#define NEW_IMAGE_VAR
#include <cv.h>
#include <vcrt.h>
#include <vclib.h>
#include <macros.h>
#include <sysvar.h>
#include <licence.h>



int main( int argc, char** argv )
{
    int old_color_mode, old_disp_start;

    IplImage *src, *h_plane, *s_plane, *v_plane, *hsv, *hist_img;      
    IplImage* planes[2];

    int h_bins = 30, s_bins = 32;
    int hist_size[2];
    float h_ranges[] = { 0, 180 }; /* hue varies from 0 (~0red) to 180 (~360red again) */
    float s_ranges[] = { 0, 255 }; /* saturation varies from 0 (black-gray-white) to 255 (pure spectrum color) */
    float* ranges[2];
    int scale = 10;

    CvHistogram* hist;
    float max_value = 0;
    int h, s;

    image rgb, bayer;


    INIT_LICENCE();
	init_VC_openCV_interface();

    src     = cvCreateImage(cvSize(ScrGetPitch, ScrGetRows), IPL_DEPTH_8U, 3);
    h_plane = cvCreateImage( cvGetSize(src), 8, 1 );
    s_plane = cvCreateImage( cvGetSize(src), 8, 1 );
    v_plane = cvCreateImage( cvGetSize(src), 8, 1 );
    hsv     = cvCreateImage( cvGetSize(src), 8, 3 );
    hist_img = cvCreateImage( cvSize(h_bins*scale,s_bins*scale), 8, 3 );

    planes[0] 		= h_plane;
    planes[1] 		= s_plane;
    hist_size[0] 	= h_bins;
    hist_size[1] 	= s_bins;
    ranges[0] 		= h_ranges;
    ranges[1] 		= s_ranges;


    /* on color cameras create RGB output image		*/

    if(getvar(COLOR_MODE) == DISP_NO_COLOR)
      {
      printf("B/W camera\n");
	  return -1;
      }
    else
      {
      ImageAssignC(&bayer, (U8 *)getvar(CAPT_START), IMAGE_BAYER, ScrGetColumns, ScrGetRows, ScrGetPitch, 0, 0);
      ImageAllocate(&rgb, IMAGE_RGB, getvar(VPITCH), getvar(DVWIDTH));		/* allocate RGB color image		*/
      rgb.dx = getvar(DHWIDTH);											    /* set dx to actual width		*/

      old_color_mode = getvar(COLOR_MODE);									/* save settings				*/
      old_disp_start = getvar(DISP_START);

      setvar(COLOR_MODE, DISP_RGB);
      setvar(DISP_START,(int)rgb.st);

      ColorBar(&rgb, 255, 255);
      }


	rs232rcv();

	/* take a picture 	*/

    tpict();

    /* convert to RGB	*/

    BayerToRGB(&bayer, &rgb);

    copyCV(&rgb, src);
 
    cvCvtColor( src, hsv, CV_BGR2HSV );
    cvCvtPixToPlane( hsv, h_plane, s_plane, v_plane, 0 );

	printf("now histogram\n"); rs232rcv();

    hist = cvCreateHist( 2, hist_size, CV_HIST_ARRAY, ranges, 1 );
    cvCalcHist( planes, hist, 0, 0 );
    cvGetMinMaxHistValue( hist, 0, &max_value, 0, 0 );
    cvZero( hist_img );

        for( h = 0; h < h_bins; h++ )
        {
            for( s = 0; s < s_bins; s++ )
            {
                float bin_val = cvQueryHistValue_2D( hist, h, s );
                int intensity = cvRound(bin_val*255/max_value);
                cvRectangle( hist_img, cvPoint( h*scale, s*scale ),
                             cvPoint( (h+1)*scale - 1, (s+1)*scale - 1),
                             CV_RGB(intensity,intensity,intensity), /* draw a grayscale histogram.
                                                                       if you have idea how to do it
                                                                       nicer let us know */
                             CV_FILLED, 8, 0 );
            }
        }

   printf("copy to VC: %d\n", copyVC(hist_img, &rgb));

   rs232rcv();

   cvReleaseImage(&src);
   cvReleaseImage(&h_plane);
   cvReleaseImage(&s_plane);
   cvReleaseImage(&v_plane);
   cvReleaseImage(&hsv);
   cvReleaseImage(&hist_img);

   if(getvar(COLOR_MODE) != DISP_NO_COLOR)
	  {
	  setvar(COLOR_MODE, old_color_mode);		/* restore settings		*/
      setvar(DISP_START, old_disp_start);
	  ImageFree(&rgb);
	  }

}



/* EOF */
