/*HEADER*******************************************************************
***************************************************************************
***
*** Copyright (c) 2008 Vision Components
***
*** All rights reserved
***
*** This software embodies materials and concepts which are confidential
*** to Vision Components
***
*** File: vcdist.c
***                                                            
*** Comments: 
*** 
***	   Test of OpenCV Distance Transformation
***
*** Revision History:
***    Date          Version    Author  Changes
***    Oct 01, 2008  3.03       ME      Initial Version
***
***************************************************************************
*END***********************************************************************/
#include "cv.h"

#define NEW_IMAGE_VAR
#include <register.h>
#include <vcrt.h>
#include <vclib.h>
#include <macros.h>
#include <sysvar.h>
#include <licence.h>

char wndname[] = "Distance transform";
char tbarname[] = "Threshold";
int mask_size = CV_DIST_MASK_5;
int build_voronoi = 0;
int edge_thresh = 100;

// The output and temporary images
IplImage* dist = 0;
IplImage* dist8u1 = 0;
IplImage* dist8u2 = 0;
IplImage* dist8u = 0;
IplImage* dist32s = 0;

IplImage* gray = 0;
IplImage* cedge = 0;
IplImage* labels = 0;


IplImage* color = 0;

image rgb;

void dist_transform( )
{
    static const uchar colors[][3] = 
    {
        {0,0,0},
        {255,0,0},
        {255,128,0},
        {255,255,0},
        {0,255,0},
        {0,128,255},
        {0,255,255},
        {0,0,255},
        {255,0,255}
    };
    
    int msize = mask_size;

    cvThreshold( gray, cedge, (float)edge_thresh, (float)edge_thresh, CV_THRESH_BINARY );

    if( build_voronoi )
        msize = CV_DIST_MASK_5;

    cvDistTransform( cedge, dist, CV_DIST_L2, msize, NULL, build_voronoi ? labels : NULL );

    if( !build_voronoi )
    {
        // begin "painting" the distance transform result
        cvConvertScale( dist, dist, 5000.0, 0 );
        cvPow( dist, dist, 0.5 );
    
        cvConvertScale( dist, dist32s, 1.0, 0.5 );
        cvAndS( dist32s, cvScalarAll(255), dist32s, 0 );
        cvConvertScale( dist32s, dist8u1, 1, 0 );
        cvConvertScale( dist32s, dist32s, -1, 0 );
        cvAddS( dist32s, cvScalarAll(255), dist32s, 0 );
        cvConvertScale( dist32s, dist8u2, 1, 0 );
        cvMerge( dist8u1, dist8u2, dist8u2, 0, dist8u );
        // end "painting" the distance transform result
    }
    else
    {
        int i, j;
        for( i = 0; i < labels->height; i++ )
        {
            int* ll = (int*)(labels->imageData + i*labels->widthStep);
            float* dd = (float*)(dist->imageData + i*dist->widthStep);
            uchar* d = (uchar*)(dist8u->imageData + i*dist8u->widthStep);
            for( j = 0; j < labels->width; j++ )
            {
                int idx = ll[j] == 0 || dd[j] == 0 ? 0 : (ll[j]-1)%8 + 1;
                int b = cvRound(colors[idx][0]);
                int g = cvRound(colors[idx][1]);
                int r = cvRound(colors[idx][2]);
                d[j*3] = (uchar)b;
                d[j*3+1] = (uchar)g;
                d[j*3+2] = (uchar)r;
            }
        }
    }
    
if(getvar(COLOR_MODE) == DISP_NO_COLOR)
  {
  cvCopy( dist8u1, gray, NULL );
  }
else
  {
  printf("copy res=%d\n", copyVC(dist8u, &rgb));
//  cvCopy( dist8u, color, NULL );
//  cvCopy( dist8u1, color, NULL );

  }


}

int main( int argc, char** argv )
{
    int old_color_mode, old_disp_start;

    INIT_LICENCE();
	init_VC_openCV_interface();

    printf("OpenCV distance transformation - hit any key\n"); rs232rcv();

/* take a picture  */

    tpict();

/* create input image header					*/

    gray = cvCreateImageHeader(cvSize(ScrGetPitch, ScrGetRows), IPL_DEPTH_8U, 1);
	cvSetImageROI( gray, cvRect(0, 0, ScrGetColumns, ScrGetRows));					/* set ROI to actual image	*/
	cvSetImageStart(gray, (char *)ScrGetPhysPage);

/* on color cameras create RGB output image		*/

if(getvar(COLOR_MODE) == DISP_NO_COLOR)
  {
  printf("B/W camera\n");
  }
else
  {
  printf("color camera\n");
  ImageAllocate(&rgb, IMAGE_RGB, getvar(VPITCH), getvar(DVWIDTH));		/* allocate RGB color image		*/
  rgb.dx = getvar(DHWIDTH);											    /* set dx to actual width		*/

  old_color_mode = getvar(COLOR_MODE);									/* save settings				*/
  old_disp_start = getvar(DISP_START);

  setvar(COLOR_MODE, DISP_RGB);
  setvar(DISP_START,(int)rgb.st);

  print("rgb:   %08x\n",(int)rgb.st);
  print("ccmp1: %08x\n",(int)rgb.ccmp1);
  print("ccmp2: %08x\n",(int)rgb.ccmp2);
  print("color test bar chart\n");

  rs232rcv();
  print("colorbar = %d\n",ColorBar(&rgb, 255, 255));
  print("end color bar\n");
  rs232rcv();

//  color = cvCreateImageHeader(cvSize(ScrGetPitch, ScrGetRows), IPL_DEPTH_8U, 3);
//  cvSetImageROI(color, cvRect(0, 0, ScrGetColumns, ScrGetRows));					/* set ROI to actual image	*/
//  cvSetImageStart(color, (char *)ScrGetPhysPage);


  color = cvCreateImage( cvGetSize(gray), IPL_DEPTH_8U, 3 );

  }


    printf( "Hot keys: \n"
        "\tESC - quit the program\n"
        "\t3 - use 3x3 mask\n"
        "\t5 - use 5x5 mask\n"
        "\t0 - use precise distance transform\n"
        "\tv - switch Voronoi diagram mode on/off\n"
        "\tENTER - loop through all the modes\n" );

    dist = cvCreateImage( cvGetSize(gray), IPL_DEPTH_32F, 1 );
    dist8u1 = cvCloneImage( gray );
    dist8u2 = cvCloneImage( gray );
    dist8u = cvCreateImage( cvGetSize(gray), IPL_DEPTH_8U, 3 );
    dist32s = cvCreateImage( cvGetSize(gray), IPL_DEPTH_32S, 1 );
    cedge = cvCloneImage( gray );
    labels = cvCreateImage( cvGetSize(gray), IPL_DEPTH_32S, 1 );


    for(;;)
    {
        int c;
        
		tpict();
        dist_transform();

		time_delay(1000);

        if(rbempty() != -1) c=rs232rcv();             /* read character       */
							else c=0;

        if( (char)c == 27 )
            break;

        if( (char)c == '3' )
            mask_size = CV_DIST_MASK_3;
        else if( (char)c == '5' )
            mask_size = CV_DIST_MASK_5;
        else if( (char)c == '0' )
            mask_size = CV_DIST_MASK_PRECISE;
        else if( (char)c == 'v' )
            build_voronoi ^= 1;
        else if( (char)c == '\n' )
        {
            if( build_voronoi )
            {
                build_voronoi = 0;
                mask_size = CV_DIST_MASK_3;
            }
            else if( mask_size == CV_DIST_MASK_3 )
                mask_size = CV_DIST_MASK_5;
            else if( mask_size == CV_DIST_MASK_5 )
                mask_size = CV_DIST_MASK_PRECISE;
            else if( mask_size == CV_DIST_MASK_PRECISE )
                build_voronoi = 1;
        }
    }

    cvReleaseImage( &cedge );
    cvReleaseImage( &dist );
    cvReleaseImage( &dist8u );
    cvReleaseImage( &dist8u1 );
    cvReleaseImage( &dist8u2 );
    cvReleaseImage( &dist32s );
    cvReleaseImage( &labels );

    cvReleaseImageHeader(&gray);


    if(getvar(COLOR_MODE) != DISP_NO_COLOR)
	  {
      cvReleaseImage(&color);
	  setvar(COLOR_MODE, old_color_mode);		/* restore settings		*/
      setvar(DISP_START, old_disp_start);
	  ImageFree(&rgb);
	  }

    
    return 0;
}








#if 0


I32 copyVC(IplImage *src, image *dst)
{
U8  *p, *q;
I32  psrc, pdst;
I32 i, dx, dy;
I32 src_class, dst_class;
I32 src_size,  dst_size;
I32 data_order;

src_class = src->nChannels;

if(src_class > 3)
  {
  printf("error, number of channels\n");
  return ERR_FORMAT;
  }

switch(src->depth)
  {
  case IPL_DEPTH_8U:
  case IPL_DEPTH_8S:  src_size  = 8;
                  	  break;
  case IPL_DEPTH_16S: src_size  = 16;
                  	  break;
  case IPL_DEPTH_32S: src_size  = 32;
                  	  break;

  default:            printf("error, pixel depth\n");
                      return ERR_TYPE;
  }                        


data_order = src->dataOrder;
   
if(data_order == IPL_DATA_ORDER_PIXEL)
  {
  printf("data order pixel\n");
  }
else
  {
  printf("data order plane\n");
  }


psrc = src->width;
pdst = dst->pitch;

dx      = min(src->width,  dst->dx);
dy      = min(src->height, dst->dy);


if(dst->type & 0x80) dst->type = 0;	/* convert old-style image types	*/

switch(dst->type)
  {
  case IMAGE_GREY: 
  case IMAGE_BAYER:   dst_class = 1;
					  dst_size  = 8;
                  	  break;
  case IMAGE_RGB: 
  case IMAGE_CBCR444:
  case IMAGE_YUVNORM:
  case IMAGE_IHS:	  dst_class = 3;
					  dst_size  = 8;
                  	  break;
  case IMAGE_CBCR411: dst_class = 2;
					  dst_size  = 8;
					  return ERR_TYPE;
  case IMAGE_VECTOR:  dst_class = 4;
					  dst_size  = 8;
                  	  break;
  case IMAGE_GREY16:  dst_class = 1;
					  dst_size  = 16;
                  	  break;
  case IMAGE_GREY32:
  case IMAGE_RGBO:    dst_class = 1;
					  dst_size  = 32;
                  	  break;
                  
  default:            return ERR_TYPE;
  }                        

if(dst_class == 4) dst_class = 2;
if(src_class != dst_class) return ERR_TYPE;
if(src_size  != dst_size)  return ERR_TYPE;


/* copy first component			*/
								 
p       = (U8 *)src->imageDataOrigin;
q       = (U8 *)dst->st;

switch(dst_size)
  {
  case 8:
		if((src_class > 1) && (data_order==IPL_DATA_ORDER_PIXEL))
		  for(i=0;i<dy;i++)
   		    {
            FL_CPY_IS_U8P_U8P(dx, src_class, p, q);
	        p += psrc*src_class; q += pdst;
            }
		else 
		  for(i=0;i<dy;i++)
   		    {
	        FL_CPY_U8P_U8P(dx, p, q);
	        p += psrc; q += pdst;
            }

         break;
   
  case 16:
		for(i=0;i<dy;i++)
   		  {
	      FL_CPY_U16P_U16P(dx, (U16 *)p, (U16 *)q);
	      p += 2*psrc; q += 2*pdst;
          }
         break;

  case 32: 
		for(i=0;i<dy;i++)
   		  {
	      FL_CPY_U32P_U32P(dx, (U32 *)p, (U32 *)q);
	      p += 4*psrc; q += 4*pdst;
          }
         break;
                
  default:            
         return ERR_TYPE;
  }                        


if(dst_class == 1) return 0;	/* we have finished so far	*/

/* now copy component 1	*/


q = dst->ccmp1;
p       = (U8 *)src->imageDataOrigin;


if(data_order==IPL_DATA_ORDER_PIXEL)
  {
  p = ((U8 *)src->imageDataOrigin) + 1;
		  
  for(i=0;i<dy;i++)
    {
    FL_CPY_IS_U8P_U8P(dx, src_class, p, q);
    p += psrc*src_class; q += pdst;
    }
  }
else 	/* p is just left from previous copy */
  for(i=0;i<dy;i++)
    {
	FL_CPY_U8P_U8P(dx, p, q);
	p += psrc; q += pdst;
    }

if(dst_class == 2) return 0;	/* we have finished so far	*/

/* now copy component 2	*/

q = dst->ccmp2;

if(data_order==IPL_DATA_ORDER_PIXEL)
  {
  p = ((U8 *)src->imageDataOrigin) + 1;
		  
  for(i=0;i<dy;i++)
    {
    FL_CPY_IS_U8P_U8P(dx, src_class, p, q);
    p += psrc*src_class; q += pdst;
    }
  }
else 	/* p is just left from previous copy */
  for(i=0;i<dy;i++)
    {
	FL_CPY_U8P_U8P(dx, p, q);
	p += psrc; q += pdst;
    }

return 0;
}


I32 copyCV(image *src, IplImage *dst)
{
U8  *p, *q;
I32  psrc, pdst;
I32 i, dx, dy;
I32 src_class, dst_class;
I32 src_size,  dst_size;
I32 data_order;

src_class = src->nChannels;

if(src_class > 3)
  {
  printf("error, number of channels\n");
  return ERR_FORMAT;
  }

switch(src->depth)
  {
  case IPL_DEPTH_8U:
  case IPL_DEPTH_8S:  src_size  = 8;
                  	  break;
  case IPL_DEPTH_16S: src_size  = 16;
                  	  break;
  case IPL_DEPTH_32S: src_size  = 32;
                  	  break;

  default:            printf("error, pixel depth\n");
                      return ERR_TYPE;
  }                        


data_order = src->dataOrder;
   
if(data_order == IPL_DATA_ORDER_PIXEL)
  {
  printf("data order pixel\n");
  }
else
  {
  printf("data order plane\n");
  }


psrc = src->width;
pdst = dst->pitch;

dx      = min(src->width,  dst->dx);
dy      = min(src->height, dst->dy);


if(dst->type & 0x80) dst->type = 0;	/* convert old-style image types	*/

switch(dst->type)
  {
  case IMAGE_GREY: 
  case IMAGE_BAYER:   dst_class = 1;
					  dst_size  = 8;
                  	  break;
  case IMAGE_RGB: 
  case IMAGE_CBCR444:
  case IMAGE_YUVNORM:
  case IMAGE_IHS:	  dst_class = 3;
					  dst_size  = 8;
                  	  break;
  case IMAGE_CBCR411: dst_class = 2;
					  dst_size  = 8;
					  return ERR_TYPE;
  case IMAGE_VECTOR:  dst_class = 4;
					  dst_size  = 8;
                  	  break;
  case IMAGE_GREY16:  dst_class = 1;
					  dst_size  = 16;
                  	  break;
  case IMAGE_GREY32:
  case IMAGE_RGBO:    dst_class = 1;
					  dst_size  = 32;
                  	  break;
                  
  default:            return ERR_TYPE;
  }                        

if(dst_class == 4) dst_class = 2;
if(src_class != dst_class) return ERR_TYPE;
if(src_size  != dst_size)  return ERR_TYPE;


/* copy first component			*/
								 
p       = (U8 *)src->imageDataOrigin;
q       = (U8 *)dst->st;

switch(dst_size)
  {
  case 8:
		if((src_class > 1) && (data_order==IPL_DATA_ORDER_PIXEL))
		  for(i=0;i<dy;i++)
   		    {
            FL_CPY_IS_U8P_U8P(dx, src_class, p, q);
	        p += psrc*src_class; q += pdst;
            }
		else 
		  for(i=0;i<dy;i++)
   		    {
	        FL_CPY_U8P_U8P(dx, p, q);
	        p += psrc; q += pdst;
            }

         break;
   
  case 16:
		for(i=0;i<dy;i++)
   		  {
	      FL_CPY_U16P_U16P(dx, (U16 *)p, (U16 *)q);
	      p += 2*psrc; q += 2*pdst;
          }
         break;

  case 32: 
		for(i=0;i<dy;i++)
   		  {
	      FL_CPY_U32P_U32P(dx, (U32 *)p, (U32 *)q);
	      p += 4*psrc; q += 4*pdst;
          }
         break;
                
  default:            
         return ERR_TYPE;
  }                        


if(dst_class == 1) return 0;	/* we have finished so far	*/

/* now copy component 1	*/


q = dst->ccmp1;
p       = (U8 *)src->imageDataOrigin;


if(data_order==IPL_DATA_ORDER_PIXEL)
  {
  p = ((U8 *)src->imageDataOrigin) + 1;
		  
  for(i=0;i<dy;i++)
    {
    FL_CPY_IS_U8P_U8P(dx, src_class, p, q);
    p += psrc*src_class; q += pdst;
    }
  }
else 	/* p is just left from previous copy */
  for(i=0;i<dy;i++)
    {
	FL_CPY_U8P_U8P(dx, p, q);
	p += psrc; q += pdst;
    }

if(dst_class == 2) return 0;	/* we have finished so far	*/

/* now copy component 2	*/

q = dst->ccmp2;

if(data_order==IPL_DATA_ORDER_PIXEL)
  {
  p = ((U8 *)src->imageDataOrigin) + 1;
		  
  for(i=0;i<dy;i++)
    {
    FL_CPY_IS_U8P_U8P(dx, src_class, p, q);
    p += psrc*src_class; q += pdst;
    }
  }
else 	/* p is just left from previous copy */
  for(i=0;i<dy;i++)
    {
	FL_CPY_U8P_U8P(dx, p, q);
	p += psrc; q += pdst;
    }

return 0;
}
#endif


/* EOF */
