/*HEADER*******************************************************************
***************************************************************************
***
*** Copyright (c) 2008 Vision Components
***
*** All rights reserved
***
*** This software embodies materials and concepts which are confidential
*** to Vision Components
***
*** File: vcharris.c
***                                                            
*** Comments: 
*** 
***	   Test of OpenCV Face Detection
***
*** Revision History:
***    Date          Version    Author  Changes
***    Oct 01, 2008  3.03       ME      Initial Version
***
***************************************************************************
*END***********************************************************************/
#include <cv.h>

#include <register.h>
#include <vcrt.h>
#include <vclib.h>
#include <macros.h>
#include <sysvar.h>

#include <math.h>
#include <float.h>
#include <limits.h>
#include <time.h>
#include <ctype.h>

#include "\ti\opencv\VC\include\vc.h"
#include <licence.h>


static CvMemStorage* storage = 0;
static CvHaarClassifierCascade* cascade = 0;

void detect_and_draw( IplImage* image );

const char* cascade_name =
    "haarcascade_frontalface_alt.xml";
/*    "haarcascade_profileface.xml";*/

int main( int argc, char** argv )
{
    IplImage *frame, *frame_copy = 0;
    int optlen = strlen("--cascade=");
    const char* input_name;
	int c;

    INIT_LICENCE();
	init_VC_openCV_interface();

	printf("face detection with openCV\n"); rs232rcv();

    if( argc > 1 && strncmp( argv[1], "--cascade=", optlen ) == 0 )
    {
		printf("option 1\n");
        cascade_name = argv[1] + optlen;
        input_name = argc > 2 ? argv[2] : 0;
    }
    else
    {
		printf("option 2\n");
//        cascade_name = "../../data/haarcascades/haarcascade_frontalface_alt2.xml";
        cascade_name = "md:/x2.xml";
        input_name = argc > 1 ? argv[1] : 0;
    }

	printf("vor cvLoad\n"); rs232rcv();
    cascade = (CvHaarClassifierCascade*)cvLoad( cascade_name, 0, 0, 0 );
    
	printf("nach cvLoad\n"); rs232rcv();

    if( !cascade )
    {
        fprintf( stderr, "ERROR: Could not load classifier cascade\n" );
        fprintf( stderr,
        "Usage: facedetect --cascade=\"<cascade_path>\" [filename|camera_index]\n" );
        return -1;
    }
    storage = cvCreateMemStorage(0);
    
/* create input image header	*/

    frame = cvCreateImageHeader(cvSize(ScrGetPitch, ScrGetRows), IPL_DEPTH_8U, 1);
	cvSetImageROI( frame, cvRect(0, 0, ScrGetPitch, ScrGetRows));					/* set ROI to actual image	*/
	cvSetImageStart(frame, (char *)ScrGetPhysPage);



    if(1)
    {
        for(;;)
        {

            tpict();


#if 0
            if( !frame_copy )
                frame_copy = cvCreateImage( cvSize(frame->width,frame->height),
                                            IPL_DEPTH_8U, frame->nChannels );
            if( frame->origin == IPL_ORIGIN_TL )
                cvCopy( frame, frame_copy, 0 );
            else
                cvFlip( frame, frame_copy, 0 );
            
            detect_and_draw( frame_copy );
#else
            detect_and_draw( frame );
#endif
            if(rbempty() != -1) c=rs232rcv();             /* read character       */
							else c=0;

            if((char)c == 27 )
               break;
        }

        cvReleaseImage( &frame_copy );
    }

#if 0 // do not load image from file
    else
    {
        const char* filename = input_name ? input_name : (char*)"lena.jpg";
        IplImage* image = cvLoadImage( filename, 1 );

        if( image )
        {
            detect_and_draw( image );
            cvWaitKey(0);
            cvReleaseImage( &image );
        }
        else
        {
            /* assume it is a text file containing the
               list of the image filenames to be processed - one per line */
            FILE* f = fopen( filename, "rt" );
            if( f )
            {
                char buf[1000+1];
                while( fgets( buf, 1000, f ) )
                {
                    int len = (int)strlen(buf);
                    while( len > 0 && isspace(buf[len-1]) )
                        len--;
                    buf[len] = '\0';
                    image = cvLoadImage( buf, 1 );
                    if( image )
                    {
                        detect_and_draw( image );
                        cvWaitKey(0);
                        cvReleaseImage( &image );
                    }
                }
                fclose(f);
            }
        }

    }
#endif
    
    return 0;
}

void detect_and_draw( IplImage* img )
{
    static CvScalar colors[] = 
    {
        {{0,0,255}},
        {{0,128,255}},
        {{0,255,255}},
        {{0,255,0}},
        {{255,128,0}},
        {{255,255,0}},
        {{255,0,0}},
        {{255,0,255}}
    };

    double scale = 4.0;		/* scale was 1.3, but 4.0 is much faster	*/

    IplImage* small_img = cvCreateImage( cvSize( cvRound (img->width/scale),
                         cvRound (img->height/scale)),
                     8, 1 );
    int i;

    cvResize( img, small_img, CV_INTER_LINEAR );
    cvEqualizeHist( small_img, small_img );
    cvClearMemStorage( storage );

    if( cascade )
    {
        CvSeq* faces = cvHaarDetectObjects( small_img, cascade, storage,
                                            1.1, 2, 0/*CV_HAAR_DO_CANNY_PRUNING*/,
                                            cvSize(30, 30) );

        for( i = 0; i < (faces ? faces->total : 0); i++ )
        {
            CvRect* r = (CvRect*)cvGetSeqElem( faces, i );
            CvPoint center;
            int radius;
            center.x = cvRound((r->x + r->width*0.5)*scale);
            center.y = cvRound((r->y + r->height*0.5)*scale);
            radius = cvRound((r->width + r->height)*0.25*scale);
            cvCircle(img, center, radius, colors[i%8], 3, 8, 0 );
        }
    }

    cvReleaseImage( &small_img );
}



/* EOF */
