/*HEADER*******************************************************************
***************************************************************************
***
*** Copyright (c) 2008 Vision Components
***
*** All rights reserved
***
*** This software embodies materials and concepts which are confidential
*** to Vision Components
***
*** File: vchough.c
***                                                            
*** Comments: 
*** 
***	   Test of OpenCV Hough Transformation
***
*** Revision History:
***    Date          Version    Author  Changes
***    Oct 01, 2008  3.03       ME      Initial Version
***
***************************************************************************
*END***********************************************************************/

/* This is a standalone program. Pass an image name as a first parameter of the program.
   Switch between standard and probabilistic Hough transform by changing "#if 1" to "#if 0" and back */

#include <math.h>
#include <cv.h>

#include <register.h>
#include <vcrt.h>
#include <vclib.h>
#include <macros.h>
#include <sysvar.h>
#include "\ti\opencv\VC\include\vc.h"
#include <licence.h>

int main(int argc, char** argv)
{
    IplImage* src;
    IplImage* dst;
    CvMemStorage* storage = cvCreateMemStorage(0);
    CvSeq* lines = 0;
    int i, ms, sec;


    INIT_LICENCE();
	init_VC_openCV_interface();

/* create input image header	*/

    src = cvCreateImageHeader(cvSize(ScrGetPitch, ScrGetRows), IPL_DEPTH_8U, 1);
	cvSetImageROI( src, cvRect(0, 0, ScrGetPitch, ScrGetRows));					/* set ROI to actual image	*/
	cvSetImageStart(src, (char *)ScrGetPhysPage);
    
    dst = cvCreateImage( cvGetSize(src), 8, 1 );
    
    printf("OpenCV Hough Transformation - hit any key\n"); rs232rcv();

    ms = getvar(MSEC);

    cvCanny( src, dst, 50, 200, 3 );

    ms = getvar(MSEC)-ms; if(ms<0) ms+=1000;
    printf("%3dms\n", ms);

    cvCopy( dst, src, NULL );
	printf("after Canny edge detection - hit key\n");rs232rcv();

	disints();
    ms  = getvar(MSEC);
	sec	= getvar(SEC);
	enaints();

#if 1
    lines = cvHoughLines2( dst, storage, CV_HOUGH_STANDARD, 1, CV_PI/180, 100, 0, 0 );
	disints();
	sec = getvar(SEC) - sec;  
	ms  = getvar(MSEC)- ms + 1000*sec;
	enaints();

    printf("%3dms\n", ms);

    for( i = 0; i < MIN(lines->total,100); i++ )
    {
        float* line = (float*)cvGetSeqElem(lines,i);
        float rho = line[0];
        float theta = line[1];
        CvPoint pt1, pt2;
        double a = cos(theta), b = sin(theta);
        double x0 = a*rho, y0 = b*rho;
        pt1.x = cvRound(x0 + 1000*(-b));
        pt1.y = cvRound(y0 + 1000*(a));
        pt2.x = cvRound(x0 - 1000*(-b));
        pt2.y = cvRound(y0 - 1000*(a));
        cvLine( src, pt1, pt2, CV_RGB(255, 255, 255), 1, CV_AA, 0 );
    }
#else
    lines = cvHoughLines2( dst, storage, CV_HOUGH_PROBABILISTIC, 1, CV_PI/180, 50, 50, 10 );
	disints();
	sec = getvar(SEC) - sec;  
	ms  = getvar(MSEC)- ms + 1000*sec;
	enaints();

    printf("%3dms\n", ms);

    for( i = 0; i < lines->total; i++ )
    {
        CvPoint* line = (CvPoint*)cvGetSeqElem(lines,i);
        cvLine( src, line[0], line[1], CV_RGB(255,255,255), 1, CV_AA, 0 );
    }
#endif


    cvReleaseImage(&dst);
    cvReleaseImageHeader(&src);

    return 0;
}

/* EOF */
