/*HEADER*******************************************************************
***************************************************************************
***
*** Copyright (c) 2008 Vision Components
***
*** All rights reserved
***
*** This software embodies materials and concepts which are confidential
*** to Vision Components
***
*** File: vckm2.c
***                                                            
*** Comments: 
*** 
***	   Test of OpenCV kMeans2 algorithm
***    currently runs on color camera only
***
*** Revision History:
***    Date          Version    Author  Changes
***    Oct 01, 2008  3.03       ME      Initial Version
***
***************************************************************************
*END***********************************************************************/
#include "cv.h"

#define NEW_IMAGE_VAR
#include <register.h>
#include <vcrt.h>
#include <vclib.h>
#include <macros.h>
#include <sysvar.h>
#include <licence.h>
#include <cv.h>


int main( int argc, char** argv )
{
    #define MAX_CLUSTERS 5
    CvScalar color_tab[MAX_CLUSTERS];
    IplImage* img = cvCreateImage( cvSize( 500, 500 ), 8, 3 );
    CvRNG rng = cvRNG(-1);
    CvPoint ipt;
    image rgb;
    int old_color_mode, old_disp_start;

    INIT_LICENCE();
	init_VC_openCV_interface();


    /* on color cameras create RGB output image		*/

    if(getvar(COLOR_MODE) == DISP_NO_COLOR)
      {
      printf("B/W camera\n");
	  return -1;
      }
    else
      {
      ImageAllocate(&rgb, IMAGE_RGB, getvar(VPITCH), getvar(DVWIDTH));		/* allocate RGB color image		*/
      rgb.dx = getvar(DHWIDTH);											    /* set dx to actual width		*/

      old_color_mode = getvar(COLOR_MODE);									/* save settings				*/
      old_disp_start = getvar(DISP_START);

      setvar(COLOR_MODE, DISP_RGB);
      setvar(DISP_START,(int)rgb.st);

      ColorBar(&rgb, 255, 255);
      }

    printf("OpenCV kMeans - hit any key\n"); rs232rcv();

    cset(&rgb, 0, 0, 0);	/* clear image	*/
    
    color_tab[0] = CV_RGB(255,0,0);
    color_tab[1] = CV_RGB(0,255,0);
    color_tab[2] = CV_RGB(100,100,255);
    color_tab[3] = CV_RGB(255,0,255);
    color_tab[4] = CV_RGB(255,255,0);

       
    for(;;)
    {
        char c;
        int k, cluster_count = cvRandInt(&rng)%MAX_CLUSTERS + 1;
        int i, sample_count = cvRandInt(&rng)%1000 + 1;
        CvMat* points = cvCreateMat( sample_count, 1, CV_32FC2 );
        CvMat* clusters = cvCreateMat( sample_count, 1, CV_32SC1 );
        
        /* generate random sample from multigaussian distribution */
        for( k = 0; k < cluster_count; k++ )
        {
            CvPoint center;
            CvMat point_chunk;
            center.x = cvRandInt(&rng)%img->width;
            center.y = cvRandInt(&rng)%img->height;
            cvGetRows( points, &point_chunk, k*sample_count/cluster_count,
                       k == cluster_count - 1 ? sample_count :
                       (k+1)*sample_count/cluster_count, 1 );
                        
            cvRandArr( &rng, &point_chunk, CV_RAND_NORMAL,
                       cvScalar(center.x,center.y,0,0),
                       cvScalar(img->width*0.1,img->height*0.1,0,0));
        }

        /* shuffle samples */
        for( i = 0; i < sample_count/2; i++ )
        {
            CvPoint2D32f* pt1 = (CvPoint2D32f*)points->data.fl + cvRandInt(&rng)%sample_count;
            CvPoint2D32f* pt2 = (CvPoint2D32f*)points->data.fl + cvRandInt(&rng)%sample_count;
            CvPoint2D32f temp;
            CV_SWAP( *pt1, *pt2, temp );
        }

        cvKMeans2( points, cluster_count, clusters,
                   cvTermCriteria( CV_TERMCRIT_EPS+CV_TERMCRIT_ITER, 10, 1.0 ));

        cvZero( img );

        for( i = 0; i < sample_count; i++ )
        {
            int cluster_idx = clusters->data.i[i];
            ipt.x = (int)points->data.fl[i*2];
            ipt.y = (int)points->data.fl[i*2+1];
            cvCircle( img, ipt, 2, color_tab[cluster_idx], CV_FILLED, CV_AA, 0 );
        }

        cvReleaseMat( &points );
        cvReleaseMat( &clusters );

        copyVC(img, &rgb);						/* copy result			*/
		time_delay(1000);						/* wait					*/

        if(rbempty() != -1) c=rs232rcv();       /* read character       */
							else c=0;		    

        if( c == 27 || c == 'q' || c == 'Q' ) // 'ESC'
            break;
    }

    if(getvar(COLOR_MODE) != DISP_NO_COLOR)
	  {
	  setvar(COLOR_MODE, old_color_mode);		/* restore settings		*/
      setvar(DISP_START, old_disp_start);
	  ImageFree(&rgb);
	  }
    
    return 0;
}

/* EOF */
