/*HEADER*******************************************************************
***************************************************************************
***
*** Copyright (c) 2008 Vision Components
***
*** All rights reserved
***
*** This software embodies materials and concepts which are confidential
*** to Vision Components
***
*** File: vcmatch.c
***                                                            
*** Comments: 
*** 
***	   Test of OpenCV pattern finder
***
*** Revision History:
***    Date          Version    Author  Changes
***    Oct 02, 2008  3.03       ME      Initial Version
***
***************************************************************************
*END***********************************************************************/

#define NEW_IMAGE_VAR
#include <cv.h>
#include <vcrt.h>
#include <vclib.h>
#include <flib.h>
#include <macros.h>
#include <sysvar.h>
#include <licence.h>
#include <math.h>

#define PX			256						/* template size	x		*/
#define PY			256						/* template size	y		*/
#define METHOD		CV_TM_CCORR_NORMED		/* normalized correlation	*/

int main( int argc, char** argv )
{
    IplImage *cinp, *covl, *ctmp, *cr32f;
    CvFont font;      
    CvSize  TextSize;

    image iinp, iovl, itmp, otmp;
    double min_val, max_val;
	CvPoint min_loc, max_loc;
	int c;

    INIT_LICENCE();
	init_VC_openCV_interface();


    printf("OpenCV pattern search - hit any key\n"); rs232rcv();

	/* create input image header												*/

    cinp = cvCreateImageHeader(cvSize(ScrGetPitch, ScrGetRows), IPL_DEPTH_8U, 1);
	cvSetImageROI(cinp, cvRect(0, 0, ScrGetPitch, ScrGetRows));					/* set ROI to actual image	*/
	cvSetImageStart(cinp, (char *)ScrGetPhysPage);								/* set start address image	*/
  	ImageAssign(&iinp, ScrGetPhysPage, ScrGetPitch,  ScrGetRows, ScrGetPitch);	/* VC image var for screen	*/

	/* create overlay image header												*/

    covl = cvCreateImageHeader(cvSize(ScrGetPitch, ScrGetRows), IPL_DEPTH_8U, 1);
	cvSetImageROI(covl, cvRect(0, 0, ScrGetPitch, ScrGetRows));					/* set ROI to overlay		*/
	cvSetImageStart(covl, (char *)OvlGetPhysPage);								/* set start address overlay*/
  	ImageAssign(&iovl, OvlGetPhysPage, OvlGetPitch,  OvlGetRows, OvlGetPitch);	/* VC image var for overlay	*/

	/* create search template, screen centered 									*/

  	ImageAssign(&itmp, ScrByteAddr((ScrGetColumns-PX)/2, (ScrGetRows-PY)/2), PX,  PY, ScrGetPitch);
  	ImageAssign(&otmp, OvlByteAddr((ScrGetColumns-PX)/2, (ScrGetRows-PY)/2), PX,  PY, OvlGetPitch);

	/* create search template image CV style, must be same size as itmp			*/

    ctmp = cvCreateImage(cvSize(itmp.dx, itmp.dy), IPL_DEPTH_8U, 1);

	/* create 32bit image for search result										*/

    cr32f = cvCreateImage(cvSize(cinp->width-itmp.dx+1, cinp->height-itmp.dy+1), IPL_DEPTH_32F, 1);

	/* clear overlay, initialize colors											*/

  	set_translucent(1, 250,   0,   0); /* TRANRED    1 red translucent 			*/
  	set_translucent(2, 255, 255,   0); /* TRANYEL    3 yellow translucent 		*/
  	set_overlay_bit(2, 210, 210, 210); /* COLGREY    4 grey 					*/
  	set_overlay_bit(3, 250,   0,   0); /* COLRED     8 red 						*/
  	set_overlay_bit(4,  70,  70,  70); /* COLBLACK  16 black 					*/
  	set_overlay_bit(5, 250, 250, 250); /* COLWHITE  32 white 					*/
  	set_overlay_bit(6,   0, 200,   0); /* COLGREEN  64 green 					*/
  	set_overlay_bit(7,  65,  90, 255); /* COLBLUE  128 blue 					*/

  	set_ovlmask(255);
	OvlClearAll;

	/* initialize text font														*/

    cvInitFont(&font, CV_FONT_HERSHEY_SCRIPT_COMPLEX , 1.0, 1.0, 0, 1, 8);

	/* teach template															*/

    vmode(vmOvlLive);															/* live + overlay			*/
	framed(&otmp, 0x08);														/* display overlay frame	*/

    cvGetTextSize("select reference pattern", &font, &TextSize, NULL);
    printf("textsize x: %d y: %d\n", TextSize.width, TextSize.height);
    cvPutText(covl, "select reference pattern", cvPoint(20, ScrGetRows-20), &font, CV_RGB(0x80, 0x80, 0x80));
    printf("select reference pattern and hit key\n"); rs232rcv();

    tpict();

	/* copy selected image to CV image variable									*/

    copyCV(&itmp, ctmp);														/* copy image to CV			*/
	cvSetZero(cinp);															/* clear input image		*/
	copyVC(ctmp, &itmp);														/* copy image to VC			*/

	/* now loop																	*/

	OvlClearAll;

    for(;;)
      {
      tpict();													  				/* take a picture 			*/

	  /* match template															*/

      cvMatchTemplate(cinp, ctmp, cr32f, METHOD);

      cvMinMaxLoc(cr32f, &min_val, &max_val,  &min_loc, &max_loc, NULL);

	  printf("x: %d y: %d val: %e\n", max_loc.x, max_loc.y, max_val);

	  OvlClearAll;
      MarkCross(&iovl, max_loc.x+PX/2, max_loc.y+PY/2, 0x40, 10);
	  time_delay(1000);

      if(rbempty() != -1) c=rs232rcv();             							/* read character       	*/
	   					  else c=0;

      if((char)c == 27)   break;												/* end on ESC				*/

	  }

    cvReleaseImage(&cr32f);
    cvReleaseImage(&ctmp);
    cvReleaseImageHeader(&covl);
    cvReleaseImageHeader(&cinp);
}



/* EOF */
