/*HEADER*******************************************************************
***************************************************************************
***
*** Copyright (c) 2008 Vision Components
***
*** All rights reserved
***
*** This software embodies materials and concepts which are confidential
*** to Vision Components
***
*** File: vcshift.c
***                                                            
*** Comments: 
*** 
***	   Test of OpenCV CAMSHIFT pattern tracker
***
*** Revision History:
***    Date          Version    Author  Changes
***    Oct 02, 2008  3.03       ME      Initial Version
***
***************************************************************************
*END***********************************************************************/

#define NEW_IMAGE_VAR
#include <cv.h>
#include <vcrt.h>
#include <vclib.h>
#include <flib.h>
#include <macros.h>
#include <sysvar.h>
#include <licence.h>
#include <math.h>

#include <ctype.h>

#define PX			256						/* template size	x		*/
#define PY			256						/* template size	y		*/

IplImage *img = 0, *hsv = 0, *hue = 0, *mask = 0, *backproject = 0, *histimg = 0;
CvHistogram *hist = 0;

int backproject_mode = 0;
int select_object = 0;
int track_object = 0;
int show_hist = 0;
CvPoint origin;
CvRect selection;
CvRect track_window;
CvBox2D track_box;
CvConnectedComp track_comp;
int hdims = 16;
float hranges_arr[] = {0,180};
float* hranges = hranges_arr;
int vmin = 50;				/* minimum grey value	*/
int vmax = 256;				/* maximum grey value	*/
int smin = 50;				/* minimum saturation	*/



CvScalar hsv2rgb( float hue )
{
    int rgb[3], p, sector;
    static const int sector_data[][3]=
        {{0,2,1}, {1,2,0}, {1,0,2}, {2,0,1}, {2,1,0}, {0,1,2}};
    hue *= 0.033333333333333333333333333333333f;
    sector = cvFloor(hue);
    p = cvRound(255*(hue - sector));
    p ^= sector & 1 ? 255 : 0;

    rgb[sector_data[sector][0]] = 255;
    rgb[sector_data[sector][1]] = 0;
    rgb[sector_data[sector][2]] = p;

    return cvScalar(rgb[2], rgb[1], rgb[0],0);
}



int main( int argc, char** argv )
{
    int old_color_mode, old_disp_start;
    IplImage* cinp = 0;
    image rgb, bayer, otmp;
    
    INIT_LICENCE();
	init_VC_openCV_interface();


	/* create search template, screen centered 									*/

  	ImageAssign(&otmp, OvlByteAddr((ScrGetColumns-PX)/2, (ScrGetRows-PY)/2), PX,  PY, OvlGetPitch);

	/* create input image header												*/

    cinp = cvCreateImageHeader(cvSize(ScrGetPitch, ScrGetRows), IPL_DEPTH_8U, 1);
	cvSetImageROI(cinp, cvRect(0, 0, ScrGetPitch, ScrGetRows));					/* set ROI to actual image	*/
	cvSetImageStart(cinp, (char *)ScrGetPhysPage);								/* set start address image	*/

    /* on color cameras create RGB output image		*/

    if(getvar(COLOR_MODE) == DISP_NO_COLOR)
      {
      printf("B/W camera\n");
	  return -1;
      }
    else
      {
      ImageAssignC(&bayer, (U8 *)getvar(CAPT_START), IMAGE_BAYER, ScrGetColumns, ScrGetRows, ScrGetPitch, 0, 0);
      ImageAllocate(&rgb, IMAGE_RGB, getvar(VPITCH), getvar(DVWIDTH));		/* allocate RGB color image		*/
      rgb.dx = getvar(DHWIDTH);											    /* set dx to actual width		*/

      old_color_mode = getvar(COLOR_MODE);									/* save settings				*/
      old_disp_start = getvar(DISP_START);

      setvar(COLOR_MODE, DISP_RGB);
      setvar(DISP_START,(int)rgb.st);

      ColorBar(&rgb, 255, 255);
      }

	/* clear overlay, initialize colors											*/

  	set_translucent(1, 250,   0,   0); /* TRANRED    1 red translucent 			*/
  	set_translucent(2, 255, 255,   0); /* TRANYEL    3 yellow translucent 		*/
  	set_overlay_bit(2, 210, 210, 210); /* COLGREY    4 grey 					*/
  	set_overlay_bit(3, 250,   0,   0); /* COLRED     8 red 						*/
  	set_overlay_bit(4,  70,  70,  70); /* COLBLACK  16 black 					*/
  	set_overlay_bit(5, 250, 250, 250); /* COLWHITE  32 white 					*/
  	set_overlay_bit(6,   0, 200,   0); /* COLGREEN  64 green 					*/
  	set_overlay_bit(7,  65,  90, 255); /* COLBLUE  128 blue 					*/

  	set_ovlmask(255);
	OvlClearAll;
    vmode(vmOvlLive);															/* live + overlay			*/
    framed(&otmp, 0x80);

    printf("OpenCV camshift demo - hit any key\n"); rs232rcv();

    printf( "Hot keys: \n"
        "\tESC - quit the program\n"
        "\tc - stop the tracking\n"
        "\tb - switch to/from backprojection view\n"
        "\th - show/hide object histogram\n"
        "To initialize tracking, select the object with mouse\n" );


    for(;;)
    {
        int i, bin_w, c;

	    /* take a picture 	*/
		tpict();

    	/* convert to RGB	*/
    	BayerToRGB(&bayer, &rgb);

        if( !img )
        {
            /* allocate all the buffers */
            img = cvCreateImage( cvGetSize(cinp), 8, 3 );
            img->origin = cinp->origin;
            hsv = cvCreateImage( cvGetSize(cinp), 8, 3 );
            hue = cvCreateImage( cvGetSize(cinp), 8, 1 );
            mask = cvCreateImage( cvGetSize(cinp), 8, 1 );
            backproject = cvCreateImage( cvGetSize(cinp), 8, 1 );
            hist = cvCreateHist( 1, &hdims, CV_HIST_ARRAY, &hranges, 1 );
            histimg = cvCreateImage( cvSize(320,200), 8, 3 );
            cvZero( histimg );
        }

        copyCV(&rgb, img);
        cvCvtColor( img, hsv, CV_BGR2HSV );

        if( track_object )
        {
            int _vmin = vmin, _vmax = vmax;

            cvInRangeS( hsv, cvScalar(0,smin,MIN(_vmin,_vmax),0),
                        cvScalar(180,256,MAX(_vmin,_vmax),0), mask );

            cvSplit( hsv, hue, 0, 0, 0 );

            if( track_object < 0 )
            {
                float max_val = 0.f;
                cvSetImageROI( hue, selection );
                cvSetImageROI( mask, selection );
                cvCalcHist( &hue, hist, 0, mask );
                cvGetMinMaxHistValue( hist, 0, &max_val, 0, 0 );
                cvConvertScale( hist->bins, hist->bins, max_val ? 255. / max_val : 0., 0 );
                cvResetImageROI( hue );
                cvResetImageROI( mask );
                track_window = selection;
                track_object = 1;

                cvZero( histimg );
                bin_w = histimg->width / hdims;
                for( i = 0; i < hdims; i++ )
                {
                    int val = cvRound( cvGetReal1D(hist->bins,i)*histimg->height/255 );
                    CvScalar color = hsv2rgb(i*180.f/hdims);
                    cvRectangle( histimg, cvPoint(i*bin_w,histimg->height),
                                 cvPoint((i+1)*bin_w,histimg->height - val),
                                 color, -1, 8, 0 );
                }
            }


            cvCalcBackProject( &hue, backproject, hist );
            cvAnd( backproject, mask, backproject, 0 );
            cvCamShift( backproject, track_window,
                        cvTermCriteria( CV_TERMCRIT_EPS | CV_TERMCRIT_ITER, 10, 1 ),
                        &track_comp, &track_box );
            track_window = track_comp.rect;
            
            if( backproject_mode )
                cvCvtColor( backproject, img, CV_GRAY2BGR );
            if( !img->origin )
                track_box.angle = -track_box.angle;
            cvEllipseBox( img, track_box, CV_RGB(255,0,0), 3, CV_AA, 0 );
            copyVC(img, &rgb);
        }
		else
		{
		/* teach template	*/
	    OvlClearAll;
  	    set_ovlmask(255);
        framed(&otmp, 0x80);								/* display overlay frame	*/
        vmode(vmOvlLive);									/* live + overlay			*/
	    setvar(COLOR_MODE, old_color_mode);					/* restore settings			*/
        setvar(DISP_START, old_disp_start);
		printf("teaching object\n"); rs232rcv();
        setvar(COLOR_MODE, DISP_RGB);
        setvar(DISP_START,(int)rgb.st);

        selection.x      = (ScrGetColumns-PX)/2;
        selection.y      = (ScrGetRows-PY)/2;
        selection.width  = otmp.dx;
        selection.height = otmp.dy;
        
        select_object = 1;
        track_object = -1;
		tpict();
  	    set_ovlmask(0);
		}
        
        if( select_object && selection.width > 0 && selection.height > 0 )
        {
            cvSetImageROI( img, selection );
            cvXorS( img, cvScalarAll(255), img, 0 );
            cvResetImageROI( img );
        }


      if(rbempty() != -1) c=rs232rcv();             							/* read character       	*/
	   					  else c=0;

        if( (char) c == 27 )
            break;

        switch( (char) c )
        {
        case 'b':
            backproject_mode ^= 1;
            break;
        case 'c':
            track_object = 0;
            cvZero( histimg );
            break;
        case 'h':
            show_hist ^= 1;
            break;
        default:
            ;
        }

            if(show_hist)
			  {
              copyVC(histimg, &rgb);
			  time_delay(1000);
			  }
    }


 
    cvReleaseImage(&histimg);
    cvReleaseHist(&hist);
    cvReleaseImage(&backproject);
    cvReleaseImage(&mask);
    cvReleaseImage(&hue);
    cvReleaseImage(&hsv);

    cvReleaseImage(&img);
    cvReleaseImageHeader(&cinp);

   if(getvar(COLOR_MODE) != DISP_NO_COLOR)
	  {
	  setvar(COLOR_MODE, old_color_mode);		/* restore settings		*/
      setvar(DISP_START, old_disp_start);
	  ImageFree(&rgb);
	  }


    return 0;
}


/* EOF */
