/*HEADER*******************************************************************
***************************************************************************
***
*** Copyright (c) 2008 Vision Components
***
*** All rights reserved
***
*** This software embodies materials and concepts which are confidential
*** to Vision Components
***
*** File: vcwater.c
***                                                            
*** Comments: 
*** 
***	   Test of OpenCV Watershed algorithm
***
*** Revision History:
***    Date          Version    Author  Changes
***    Oct 01, 2008  3.03       ME      Initial Version
***
***************************************************************************
*END***********************************************************************/
#include <cv.h>

#define NEW_IMAGE_VAR
#include <register.h>
#include <vcrt.h>
#include <vclib.h>
#include <macros.h>
#include <sysvar.h>
#include <licence.h>


IplImage* marker_mask = 0;
IplImage* markers = 0;
IplImage* cinp = 0, *img = 0, *img_gray = 0, *wshed = 0;
CvPoint prev_pt = {-1,-1};


int main( int argc, char** argv )
{
    CvRNG rng = cvRNG(-1);

    INIT_LICENCE();
	init_VC_openCV_interface();

    /* create input image header		*/

    cinp = cvCreateImageHeader(cvSize(ScrGetPitch, ScrGetRows), IPL_DEPTH_8U, 1);
	cvSetImageROI( cinp, cvRect(0, 0, ScrGetPitch, ScrGetRows));					/* set ROI to actual image	*/
	cvSetImageStart(cinp, (char *)ScrGetPhysPage);

    printf("OpenCV Watershed transformation - hit any key\n"); rs232rcv();

	/* take a picture					*/

    tpict();


    printf( "Hot keys: \n"
            "\tESC - quit the program\n"
            "\tr - restore the original image\n"
            "\tw or ENTER - run watershed algorithm\n"
            "\t\t(before running it, roughly mark the areas on the image)\n"
            "\t  (before that, roughly outline several markers on the image)\n" );
    
    img = cvCreateImage(cvGetSize(cinp), 8, 3 );
    img_gray = cvCreateImage(cvGetSize(img), 8, 3 );
    wshed = cvCreateImage(cvGetSize(cinp), 8, 3 );

    cvCvtColor( cinp, img,   CV_GRAY2BGR );
    cvCvtColor( cinp, wshed, CV_GRAY2BGR );

    marker_mask = cvCreateImage( cvGetSize(img), 8, 1 );
    markers = cvCreateImage( cvGetSize(img), IPL_DEPTH_32S, 1 );
    cvCvtColor( img, marker_mask, CV_BGR2GRAY );
    cvCvtColor( marker_mask, img_gray, CV_GRAY2BGR );

    cvZero( marker_mask );
    cvZero( wshed );

    cvCvtColor( img, cinp, CV_BGR2GRAY );
    cvCvtColor( wshed, cinp, CV_BGR2GRAY );


	printf("before loop\n");


    cvLine( marker_mask, cvPoint(10,10), cvPoint(100,100), cvScalarAll(255), 5, 8, 0 );
    cvLine( img, cvPoint(10,10), cvPoint(100,100), cvScalarAll(255), 5, 8, 0 );

    cvCvtColor( img, cinp, CV_BGR2GRAY );

    for(;;)
    {
	    int c;

        if(rbempty() != -1) c=rs232rcv();             /* read character       */
							else c=0;

        if( (char)c == 27 )
            break;

        if( (char)c == 'r' )
        {
            cvZero( marker_mask );
            cvCopy( cinp, img, NULL );		/* ?? */
        }

        if( (char)c == 'w' || (char)c == '\n' )
        {
            CvMemStorage* storage = cvCreateMemStorage(0);
            CvSeq* contours = 0;
            CvMat* color_tab;
            int i, j, comp_count = 0;
 
			printf("w detected\n");
            cvFindContours( marker_mask, storage, &contours, sizeof(CvContour),
                            CV_RETR_CCOMP, CV_CHAIN_APPROX_SIMPLE, cvPoint(0,0));
            cvZero( markers );
            for( ; contours != 0; contours = contours->h_next, comp_count++ )
            {
                cvDrawContours( markers, contours, cvScalarAll(comp_count+1),
                                cvScalarAll(comp_count+1), -1, -1, 8, cvPoint(0,0) );
            }

            color_tab = cvCreateMat( 1, comp_count, CV_8UC3 );
            for( i = 0; i < comp_count; i++ )
            {
                uchar* ptr = color_tab->data.ptr + i*3;
                ptr[0] = (uchar)(cvRandInt(&rng)%180 + 50);
                ptr[1] = (uchar)(cvRandInt(&rng)%180 + 50);
                ptr[2] = (uchar)(cvRandInt(&rng)%180 + 50);
            }

            {
            double t = (double)cvGetTickCount();
            cvWatershed(img, markers);
            t = (double)cvGetTickCount() - t;
            printf( "exec time = %gms\n", t/(cvGetTickFrequency()*1000.) );
            }

            // paint the watershed image
            for( i = 0; i < markers->height; i++ )
                for( j = 0; j < markers->width; j++ )
                {
                    int idx = CV_IMAGE_ELEM( markers, int, i, j );
                    uchar* dst = &CV_IMAGE_ELEM( wshed, uchar, i, j*3 );
                    if( idx == -1 )
                        dst[0] = dst[1] = dst[2] = (uchar)255;
                    else if( idx <= 0 || idx > comp_count )
                        dst[0] = dst[1] = dst[2] = (uchar)0; // should not get here
                    else
                    {
                        uchar* ptr = color_tab->data.ptr + (idx-1)*3;
                        dst[0] = ptr[0]; dst[1] = ptr[1]; dst[2] = ptr[2];
                    }
                }

            cvAddWeighted( wshed, 0.5, img_gray, 0.5, 0, wshed );
//            cvCopy(wshed, cinp, NULL );						/* copy result			*/
            cvCvtColor( wshed, cinp, CV_BGR2GRAY );
            cvReleaseMemStorage( &storage );
            cvReleaseMat( &color_tab );
        }
    }


    return 1;
}


/* EOF */
